/**
 * DevExtreme (ui/selection/selection.strategy.deferred.js)
 * Version: 16.2.2
 * Build date: Wed Nov 23 2016
 *
 * Copyright (c) 2012 - 2016 Developer Express Inc. ALL RIGHTS RESERVED
 * EULA: https://www.devexpress.com/Support/EULAs/DevExtreme.xml
 */
"use strict";
var $ = require("jquery"),
    commonUtils = require("../../core/utils/common"),
    SelectionStrategy = require("./selection.strategy"),
    errors = require("../widget/ui.errors.js"),
    dataQuery = require("../../data/query");
module.exports = SelectionStrategy.inherit({
    getSelectedItems: function() {
        return this._selection._loadFilteredData(this._selection._selectionFilter)
    },
    getSelectedItemKeys: function() {
        var d = $.Deferred(),
            dataSource = this._selection._dataGetter.dataSource(),
            store = dataSource && dataSource.store(),
            key = store && store.key(),
            select = commonUtils.isString(key) ? [key] : key;
        this._selection._loadFilteredData(this._selection._selectionFilter, null, select).done(function(items) {
            var keys = items.map(function(item) {
                return store.keyOf(item)
            });
            d.resolve(keys)
        }).fail(d.reject);
        return d.promise()
    },
    selectedItemKeys: function(keys, preserve, isDeselect, isSelectAll) {
        if (isSelectAll) {
            var filter = this._selection._dataGetter.filter();
            if (!filter) {
                this._selection._selectionFilter = isDeselect ? [] : null
            } else {
                this._selection._addSelectionFilter(isDeselect, filter, false)
            }
        } else {
            if (!preserve) {
                this._selection._selectionFilter = []
            }
            for (var i = 0; i < keys.length; i++) {
                if (isDeselect) {
                    this.removeSelectedItem(keys[i])
                } else {
                    this.addSelectedItem(keys[i])
                }
            }
        }
        this._selection.onSelectionChanged();
        return $.Deferred().resolve()
    },
    setSelectedItems: function(keys) {
        this._selection._selectionFilter = null;
        for (var i = 0; i < keys.length; i++) {
            this.addSelectedItem(keys[i])
        }
    },
    isItemDataSelected: function(itemData) {
        return this.isItemKeySelected(itemData)
    },
    isItemKeySelected: function(itemData) {
        var selectionFilter = this._selection._selectionFilter;
        if (!selectionFilter) {
            return true
        }
        return !!dataQuery([itemData]).filter(selectionFilter).toArray().length
    },
    _processSelectedItem: function(key) {
        var dataSource = this._selection._dataGetter.dataSource(),
            keyField = dataSource && dataSource.store().key(),
            filter = [keyField, "=", key];
        if ($.isArray(keyField)) {
            filter = [];
            for (var i = 0; i < keyField.length; i++) {
                filter.push([keyField[i], "=", key[keyField[i]]]);
                if (i !== keyField.length - 1) {
                    filter.push("and")
                }
            }
        }
        return filter
    },
    addSelectedItem: function(key) {
        var filter = this._processSelectedItem(key);
        this._selection._addSelectionFilter(false, filter, true)
    },
    removeSelectedItem: function(key) {
        var filter = this._processSelectedItem(key);
        this._selection._addSelectionFilter(true, filter, true)
    },
    validate: function() {
        var store = this._selection._dataGetter.store();
        if (store && !store.key()) {
            throw errors.Error("E1042")
        }
    },
    _isLastSubFilter: function(selectionFilter, filter) {
        if (selectionFilter && filter) {
            return this._selection._findSubFilter(selectionFilter, filter) === selectionFilter.length - 1
        }
        return false
    },
    getSelectAllState: function() {
        var filter = this._selection._dataGetter.filter(),
            selectionFilter = this._selection._selectionFilter;
        if (!selectionFilter) {
            return true
        }
        if (!selectionFilter.length) {
            return false
        }
        if (!filter || !filter.length) {
            return
        }
        selectionFilter = this._selection._denormalizeFilter(selectionFilter);
        if (this._isLastSubFilter(selectionFilter, filter)) {
            return true
        }
        if (this._isLastSubFilter(selectionFilter, ["!", filter])) {
            return false
        }
        return
    }
});
