/**
 * DevExtreme (ui/selection/selection.strategy.standard.js)
 * Version: 16.2.2
 * Build date: Wed Nov 23 2016
 *
 * Copyright (c) 2012 - 2016 Developer Express Inc. ALL RIGHTS RESERVED
 * EULA: https://www.devexpress.com/Support/EULAs/DevExtreme.xml
 */
"use strict";
var commonUtils = require("../../core/utils/common"),
    SelectionStrategy = require("./selection.strategy");
module.exports = SelectionStrategy.inherit({
    getSelectedItemKeys: function() {
        return this._selection._selectedItemKeys.slice(0)
    },
    getSelectedItems: function() {
        return this._selection._selectedItems.slice(0)
    },
    selectedItemKeys: function(keys, preserve, isDeselect, isSelectAll) {
        var selection = this._selection,
            deferred = selection._loadSelectedItems(keys, isDeselect, isSelectAll);
        deferred.done(function(items) {
            if (preserve) {
                selection._preserveSelectionUpdate(items, isDeselect)
            } else {
                selection._replaceSelectionUpdate(items)
            }
            if (!isSelectAll && !isDeselect) {
                selection._warnOnIncorrectKeys(keys)
            }
            selection.onSelectionChanged()
        });
        return deferred
    },
    addSelectedItem: function(key, itemData) {
        var keyHash = this._selection._getKeyHash(key);
        if (this._selection._indexOfSelectedItemKey(keyHash) === -1) {
            if (!commonUtils.isObject(keyHash) && this._selection._keyHashIndices) {
                this._selection._keyHashIndices[keyHash] = [this._selection._selectedItemKeys.length]
            }
            this._selection._selectedItemKeys.push(key);
            this._selection._addedItemKeys.push(key);
            this._selection._selectedItems.push(itemData)
        }
    },
    removeSelectedItem: function(key) {
        var keyHash = this._selection._getKeyHash(key),
            keyIndex = this._selection._indexOfSelectedItemKey(keyHash);
        if (keyIndex < 0) {
            return
        }
        this._selection._selectedItemKeys.splice(keyIndex, 1);
        this._selection._removedItemKeys.push(key);
        this._selection._selectedItems.splice(keyIndex, 1);
        if (commonUtils.isObject(keyHash) || !this._selection._keyHashIndices) {
            return
        }
        var keyIndices = this._selection._keyHashIndices[keyHash];
        if (!keyIndices) {
            return
        }
        keyIndices.shift();
        if (!keyIndices.length) {
            delete this._selection._keyHashIndices[keyHash]
        }
        this._selection._shiftSelectedKeyIndices(keyIndex)
    },
    setSelectedItems: function(keys, items) {
        this._selection._updateRemovedItemKeys(keys);
        this._selection._updateAddedItemKeys(keys);
        if (!this._selection._equalByReference) {
            this._selection._initSelectedItemKeyHash();
            this._selection._updateSelectedItemKeyHash(keys)
        }
        this._selection._selectedItemKeys = keys;
        this._selection._selectedItems = items
    },
    isItemDataSelected: function(itemData) {
        var key = this._selection._dataGetter.store().keyOf(itemData);
        return this.isItemKeySelected(key)
    },
    isItemKeySelected: function(key) {
        var keyHash = this._selection._getKeyHash(key);
        var index = this._selection._indexOfSelectedItemKey(keyHash);
        return index !== -1
    },
    getSelectAllState: function(visibleOnly) {
        if (visibleOnly) {
            return this._getVisibleSelectAllState()
        } else {
            return this._getFullSelectAllState()
        }
    },
    onSelectionChanged: function() {
        if (!this._selection._addedItemKeys.length && !this._selection._removedItemKeys.length) {
            return
        }
        this.callBase()
    }
});
